
/* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
 * All rights reserved.
 *
 * This package is an SSL implementation written
 * by Eric Young (eay@cryptsoft.com).
 * The implementation was written so as to conform with Netscapes SSL.
 * 
 * This library is free for commercial and non-commercial use as long as
 * the following conditions are aheared to.  The following conditions
 * apply to all code found in this distribution, be it the RC4, RSA,
 * lhash, DES, etc., code; not just the SSL code.  The SSL documentation
 * included with this distribution is covered by the same copyright terms
 * except that the holder is Tim Hudson (tjh@cryptsoft.com).
 * 
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * If this package is used in a product, Eric Young should be given attribution
 * as the author of the parts of the library used.
 * This can be in the form of a textual message at program startup or
 * in documentation (online or textual) provided with the package.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    "This product includes cryptographic software written by
 *     Eric Young (eay@cryptsoft.com)"
 *    The word 'cryptographic' can be left out if the rouines from the library
 *    being used are not cryptographic related :-).
 * 4. If you include any Windows specific code (or a derivative thereof) from 
 *    the apps directory (application code) you must include an acknowledgement:
 *    "This product includes software written by Tim Hudson (tjh@cryptsoft.com)"
 * 
 * THIS SOFTWARE IS PROVIDED BY ERIC YOUNG ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed.  i.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <openssl/crypto.h>
#include <openssl/bio.h>
#include <openssl/bn.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#include <openssl/dh.h>

#include "tests.h"

static const unsigned char BNp_1024[] = {
        0xC1, 0xF1, 0x54, 0x26, 0x9C, 0x53, 0xC7, 0x22,
        0x29, 0xBC, 0x03, 0x29, 0x00, 0x73, 0x61, 0xA7,
        0x20, 0x7A, 0x1E, 0x75, 0x6C, 0x01, 0xCA, 0x8D,
        0xE8, 0x9B, 0x4E, 0x1C, 0xC7, 0x59, 0x14, 0x35,
        0x63, 0x29, 0x56, 0x09, 0x57, 0xD8, 0x01, 0x2F,
        0xAD, 0x8F, 0x16, 0x78, 0x78, 0x9D, 0xA1, 0x03,
        0x74, 0x5F, 0xAC, 0x7A, 0x37, 0x76, 0x12, 0x50,
        0xF4, 0xBB, 0x4C, 0xCE, 0xD8, 0xF7, 0x87, 0x31,
        0xD9, 0x45, 0xFC, 0xB6, 0xBF, 0xD5, 0x08, 0xFB,
        0xA9, 0xBC, 0xA4, 0xA8, 0x65, 0x77, 0x69, 0xA4,
        0x89, 0xD4, 0xB4, 0x40, 0x58, 0xCC, 0xF8, 0x62,
        0x1A, 0x3E, 0x7B, 0x7A, 0x0E, 0x01, 0xA5, 0x8B,
        0x1D, 0x5B, 0xED, 0xB1, 0x0D, 0x4A, 0x44, 0x70,
        0x3B, 0xE7, 0x93, 0x4A, 0xC8, 0xFB, 0xAE, 0x81,
        0x77, 0xC0, 0x23, 0x7D, 0xBB, 0x96, 0x72, 0xF7,
        0x42, 0xFF, 0x80, 0xD6, 0x87, 0xE5, 0x97, 0xCB};

static const unsigned char BNp_2048[]= {
        0xFC, 0x90, 0x16, 0xE9, 0x4B, 0xE4, 0x28, 0x49,
        0xC2, 0xDF, 0x6E, 0xFC, 0xF4, 0xB1, 0xC5, 0xBC,
        0x48, 0xD1, 0xEC, 0x3C, 0xE6, 0xEA, 0xDC, 0x8E,
        0x5B, 0x51, 0x8A, 0x48, 0xCF, 0xC9, 0xB1, 0xAC,
        0x29, 0x30, 0x45, 0x22, 0x07, 0x71, 0x6C, 0x05,
        0x2D, 0xE8, 0xEE, 0x46, 0x16, 0xC3, 0x8B, 0xFE,
        0x65, 0xF5, 0xDA, 0x68, 0x76, 0x4F, 0x0D, 0x05,
        0x15, 0x9B, 0x21, 0xF6, 0x1C, 0x92, 0xCD, 0xFE,
        0xBE, 0x2F, 0xD0, 0x79, 0xA5, 0x6C, 0x23, 0xC6,
        0x00, 0xC9, 0xBB, 0xD1, 0x63, 0xCD, 0x06, 0x21,
        0xD1, 0xD3, 0x55, 0x2F, 0x1E, 0x38, 0x5A, 0xCC,
        0x49, 0x85, 0x13, 0xFE, 0xA0, 0x55, 0x1F, 0x51,
        0xE3, 0xE3, 0xA2, 0x3F, 0x00, 0x04, 0xAB, 0xB4,
        0xF3, 0x5A, 0x53, 0x82, 0xD0, 0xD5, 0xD7, 0x5A,
        0x11, 0xE5, 0xE3, 0xDB, 0x57, 0xA8, 0xB3, 0xB8,
        0x70, 0xF8, 0x6E, 0xEC, 0xC0, 0x46, 0x82, 0xEC,
        0x98, 0xBD, 0xE0, 0xD2, 0xB2, 0x5E, 0x44, 0x51,
        0x41, 0x18, 0x66, 0x93, 0xF1, 0xA5, 0xF3, 0x86,
        0xC9, 0x9D, 0xEC, 0x80, 0x78, 0x0B, 0x7E, 0x13,
        0xF9, 0xDF, 0x78, 0x0E, 0x0D, 0x23, 0xBB, 0x6A,
        0xF2, 0x9C, 0xBF, 0x2F, 0xD3, 0xF1, 0xFA, 0xC4,
        0xA1, 0xEB, 0xF3, 0x53, 0xBD, 0x4C, 0xDF, 0x8C,
        0xBB, 0x4A, 0x41, 0x90, 0x54, 0x72, 0x8B, 0x8D,
        0xAA, 0xAF, 0x3B, 0x2C, 0x52, 0x6F, 0x27, 0x0E,
        0xDD, 0x82, 0xF8, 0xC8, 0x72, 0xDB, 0xE1, 0x6A,
        0x81, 0x5D, 0xA4, 0xE2, 0x45, 0xA6, 0xA6, 0x7D,
        0x7B, 0x82, 0x08, 0x8B, 0x03, 0xD2, 0x70, 0xD3,
        0x9B, 0x9B, 0x05, 0x6D, 0x63, 0x1F, 0x37, 0x32,
        0xCF, 0xAE, 0xA9, 0x4B, 0xAD, 0x50, 0xB0, 0xF1,
        0xA1, 0x75, 0x4B, 0xF9, 0xEF, 0x38, 0x18, 0x94,
        0x2C, 0x30, 0x26, 0x0D, 0x63, 0x70, 0x24, 0xED,
        0x7D, 0x2F, 0xDD, 0x96, 0xD2, 0x7E, 0xD1, 0xD7};


static const unsigned char BNp_4096[]= {
        0xBF, 0x01, 0x78, 0x3A, 0x05, 0xC6, 0x8E, 0xFA,
        0x4E, 0x42, 0x93, 0xFC, 0x2F, 0x80, 0x26, 0x09,
        0xC5, 0x59, 0x7A, 0xC6, 0x5E, 0x15, 0xD8, 0xBF,
        0x46, 0x0F, 0xAB, 0x10, 0xD9, 0x01, 0x3A, 0x76,
        0x32, 0xCE, 0xB6, 0x8D, 0x7D, 0x10, 0x7E, 0xA3,
        0xE1, 0x12, 0x9F, 0xCF, 0x64, 0x54, 0xB9, 0x38,
        0x96, 0x0D, 0x2B, 0xDC, 0xE8, 0x1D, 0x6E, 0x34,
        0x45, 0xAE, 0x4E, 0x49, 0x36, 0x5C, 0x33, 0x1E,
        0x1E, 0xE4, 0xB5, 0x76, 0x87, 0x19, 0x82, 0x4B,
        0xDA, 0xC6, 0x0F, 0x5B, 0x28, 0x8A, 0x82, 0xB7,
        0x2D, 0x3A, 0xDC, 0x0C, 0x68, 0x0B, 0xA5, 0xE6,
        0x69, 0xCC, 0x9F, 0x87, 0x19, 0xFE, 0x65, 0xDE,
        0xFE, 0x6A, 0xDD, 0xE2, 0x68, 0x08, 0x2B, 0x79,
        0x4D, 0xA7, 0x23, 0x7C, 0x96, 0x92, 0x8B, 0x3A,
        0x6E, 0x36, 0x64, 0xD0, 0x91, 0x01, 0xDC, 0x7B,
        0x7B, 0x44, 0x2A, 0xAD, 0xA2, 0x26, 0x6D, 0x3A,
        0xD5, 0xF6, 0xEA, 0x56, 0x58, 0x4D, 0xA7, 0x4E,
        0xC7, 0x8C, 0x4B, 0x61, 0x9C, 0x9F, 0x1F, 0xFD,
        0x65, 0xC2, 0xA8, 0xAC, 0x94, 0x8A, 0xF8, 0x65,
        0x73, 0x08, 0x5A, 0x27, 0x6E, 0x0B, 0xE3, 0xAF,
        0xB5, 0x19, 0xEA, 0x22, 0xFA, 0x91, 0x5A, 0x65,
        0xD1, 0x5A, 0x28, 0x4C, 0xF3, 0x06, 0xAA, 0x37,
        0xDD, 0xB7, 0x19, 0x38, 0xCD, 0x5B, 0xED, 0x0D,
        0x9C, 0x59, 0xED, 0x02, 0x44, 0x3C, 0x02, 0xC2,
        0xEC, 0xAF, 0xCF, 0x62, 0xC8, 0xD4, 0x36, 0x78,
        0x68, 0x43, 0x86, 0x6F, 0x3F, 0xFD, 0x58, 0x4D,
        0x15, 0x86, 0xCB, 0xF3, 0x3E, 0xE0, 0xEF, 0x2F,
        0xD5, 0x5E, 0x3A, 0x81, 0xAB, 0x6D, 0x85, 0xD2,
        0x8E, 0x6D, 0x84, 0x35, 0xC8, 0xFD, 0x42, 0x3A,
        0xCB, 0x9A, 0x5C, 0x25, 0xC6, 0x92, 0x60, 0xAB,
        0xED, 0xC3, 0x04, 0x6E, 0xDE, 0x7F, 0xD4, 0x68,
        0xC9, 0xAE, 0xB6, 0xC1, 0xF7, 0xCE, 0x28, 0x2C,
        0xE7, 0x66, 0xA1, 0xF9, 0x62, 0x50, 0x3C, 0x20,
        0x57, 0x1E, 0x7F, 0x74, 0xB8, 0x52, 0x42, 0xB8,
        0x70, 0xFF, 0x32, 0x46, 0x8C, 0x34, 0xC5, 0x6A,
        0x11, 0x62, 0x1E, 0x03, 0xFB, 0x79, 0x1F, 0x9B,
        0xAC, 0xAF, 0xED, 0x9E, 0x98, 0xC8, 0xF9, 0x4F,
        0x68, 0x85, 0xD3, 0x3C, 0x07, 0xE9, 0xBB, 0x41,
        0x67, 0x0C, 0x79, 0x28, 0x3F, 0xF5, 0x8B, 0x8E,
        0xAC, 0xB5, 0x80, 0x40, 0x0C, 0x90, 0x33, 0x68,
        0xE3, 0x4F, 0x97, 0xE7, 0x94, 0x22, 0x20, 0x80,
        0x01, 0x1A, 0x9F, 0x1F, 0x6F, 0x22, 0x06, 0x05,
        0xC7, 0x86, 0x85, 0x1B, 0xF8, 0x7A, 0xB0, 0xF6,
        0xFB, 0xA5, 0x59, 0xF8, 0xA5, 0xFB, 0xC1, 0x88,
        0xF9, 0x33, 0xF0, 0x38, 0xBB, 0xF0, 0x14, 0x1C,
        0xFE, 0xAF, 0x14, 0x9A, 0xCF, 0xBE, 0xF4, 0xF6,
        0x9C, 0x73, 0xE4, 0xF4, 0x81, 0x41, 0x30, 0x8C,
        0x53, 0xA9, 0x17, 0xA2, 0x8C, 0xF9, 0xA4, 0x0C,
        0xFD, 0x29, 0x26, 0x49, 0x70, 0x20, 0xFB, 0xB5,
        0x34, 0x5C, 0x04, 0x86, 0x49, 0x4C, 0xFF, 0x8D,
        0x45, 0x25, 0xFB, 0x80, 0x37, 0x54, 0xCC, 0x74,
        0xCC, 0xBF, 0x39, 0x3B, 0x9A, 0x43, 0xF6, 0x44,
        0xEB, 0x0B, 0x88, 0xE8, 0xFF, 0x9F, 0x82, 0xCE,
        0x6F, 0x0A, 0x16, 0x7E, 0x5A, 0x50, 0x34, 0x8A,
        0x8A, 0x39, 0x19, 0x4D, 0x1B, 0x1F, 0x4E, 0x1D,
        0x7A, 0x12, 0x9A, 0x5B, 0x29, 0xD0, 0x3D, 0xAA,
        0x2D, 0x4B, 0x59, 0xCA, 0x29, 0xC3, 0x7F, 0xF9,
        0xD7, 0x36, 0x74, 0xE7, 0x86, 0x11, 0x83, 0x7C,
        0xDD, 0x45, 0xC8, 0x27, 0x39, 0xC1, 0x9D, 0x2A,
        0xDB, 0x57, 0xA4, 0x5B, 0x8F, 0x50, 0x4A, 0x3D,
        0xFD, 0x98, 0x45, 0xE6, 0x33, 0x50, 0xA1, 0xF4,
        0xF7, 0x4D, 0xB7, 0x99, 0x7E, 0x04, 0x3E, 0xC7,
        0x9E, 0xF0, 0x4C, 0xA9, 0xB3, 0x25, 0x88, 0xE3,
        0x25, 0xCA, 0x2A, 0x6F, 0xDD, 0xD3, 0xE3, 0xBF};

static const unsigned char BNg[] ={0x05};


/******************************************************************************
* function:
*           run_dh (int count,
*                   ENGINE *e
*                   int print_output
*                   int verify)
*
* @param count [IN] - number of iterations
* @param size [IN] - prime number exponent size 
* @param e [IN] - OpenSLL engine pointer
* @param print_output [IN] - print hex output flag
* @param verify [IN] - verify flag
*
* description:
*   The function is design to test DH key generation and verify using qat engine. 
*   If the verify flag raised, two DH public keys are generate separately 
*   with and without DH_FLAG_NO_EXP_CONSTTIME flag. And the verification compares 
*   the outputs of each DH_compute_key() after exchange two public keys.  
*
********************************************************************************/
void run_dh(int count, int size, ENGINE * e, int print_output, int verify)
{
    DH *dh_a = NULL;
    DH *dh_b = NULL;
    char buf[12] = { 0, };
    unsigned char *abuf = NULL, *bbuf = NULL;
    int i = 0, alen = 0, blen = 0, aout = 0, bout = 0, ret = 0;

    BIO *out = NULL;

    out = BIO_new(BIO_s_file());
    if (out == NULL)
        goto err;
    BIO_set_fp(out, stdout, BIO_NOCLOSE);

    /* callback function cannot be NULL */

    if ((dh_a = DH_new()) == NULL)
        goto err;
    
    if(size <= 1024)
	dh_a->p = BN_bin2bn(BNp_1024, sizeof(BNp_1024), dh_a->p);
    if(size > 1024 && size <= 2048)
	dh_a->p = BN_bin2bn(BNp_2048, sizeof(BNp_2048), dh_a->p);
    if(size > 2048)
	dh_a->p = BN_bin2bn(BNp_4096, sizeof(BNp_4096), dh_a->p);
    
    dh_a->g = BN_bin2bn(BNg, sizeof(BNg), dh_a->g);
    dh_b = DH_new();
    if (dh_b == NULL)
        goto err;

    /* we copy over prime and generator values from dh_a, just save time to
       generate and validate values */
    dh_b->p = BN_dup(dh_a->p);
    dh_b->g = BN_dup(dh_a->g);
    if ((dh_b->p == NULL) || (dh_b->g == NULL))
        goto err;

    /* Set dh_a to run with normal modexp and dh_b to use constant time The
       build-in DH implementation now uses constant time modular
       exponentiation for secret exponents by default. DH_FLAG_NO_EXP_CONTTIME
       flag causes the faster variable sliding window method to be used for
       all exponents */
    dh_b->flags &= ~DH_FLAG_NO_EXP_CONSTTIME;
    dh_a->flags |= DH_FLAG_NO_EXP_CONSTTIME;
  
        if (!DH_generate_key(dh_a))
            goto err;

        if (print_output)
        {
            BIO_puts(out, "pri 1=");
            BN_print(out, dh_a->priv_key);
            BIO_puts(out, "\npub 1=");
            BN_print(out, dh_a->pub_key);
            BIO_puts(out, "\n");
        }
  
        if (!DH_generate_key(dh_b))
            goto err;

        if (print_output)
        {
            BIO_puts(out, "pri 2=");
            BN_print(out, dh_b->priv_key);
            BIO_puts(out, "\npub 2=");
            BN_print(out, dh_b->pub_key);
            BIO_puts(out, "\n");
        }

        alen = DH_size(dh_a);
        abuf = (unsigned char *)OPENSSL_malloc(alen);
        if (abuf == NULL)
        {
            printf("abuf malloc failed ! \n");
            goto err;
        }

    for (i = 0; i < count; i++)
    {
        aout = DH_compute_key(abuf, dh_b->pub_key, dh_a);
    

        if (print_output)
        {
            BIO_puts(out, "key1 =");
            for (i = 0; i < aout; i++)
            {
                sprintf(buf, "%02X", abuf[i]);
                BIO_puts(out, buf);
            }
            BIO_puts(out, "\n");
        }
    } 
    if(verify){ 
        blen = DH_size(dh_b);
        bbuf = (unsigned char *)OPENSSL_malloc(blen);
        if (bbuf == NULL)
        {
            printf("bbuf malloc failed ! \n");
            goto err;
        }

        bout = DH_compute_key(bbuf, dh_a->pub_key, dh_b);
        if (print_output)
        {
            BIO_puts(out, "key2 =");
            for (i = 0; i < bout; i++)
            {
                sprintf(buf, "%02X", bbuf[i]);
                BIO_puts(out, buf);
            }
            BIO_puts(out, "\n");
        }

        if ((aout < 4) || (bout != aout) || (memcmp(abuf, bbuf, aout) != 0))
        {
            ret = 1;
            printf("# FAIL verify for DH.\n");
        }
    }
    if (!ret)
        printf("# PASS verify for DH. \n");
  err:
    ERR_print_errors_fp(stderr);
    if (abuf != NULL)
        OPENSSL_free(abuf);
    if (bbuf != NULL)
        OPENSSL_free(bbuf);
    if (dh_b != NULL)
        DH_free(dh_b);
    if (dh_a != NULL)
        DH_free(dh_a);
    BIO_free(out);

}

/******************************************************************************
* function:
*       tests_run_dh (int count,
*                     int size,
*		   engine *e	
*                     int print_output
*                     int verify)
*
* @param count [IN] - number of iterations
* @param size [IN] - the length of input message
* @param e [IN] - OpenSLL engine pointer
* @param print_output [IN] - print hex output flag
* @param verify [IN] - verify flag
*
* description:
*	specify a test case
*
******************************************************************************/
void tests_run_dh(int count, int size, ENGINE * e, int print_output, int verify)
{
    run_dh(count, size, e, print_output, verify);
}
